@@ -20,8 +20,8 @@ import os
 import re
 import subprocess
 import sys
+import inquirer
 
-from prettytable import PrettyTable
 from datetime import datetime, timedelta
 from pathlib import Path
 from configparser import ConfigParser
@@ -30,10 +30,18 @@ from dateutil.parser import parse
 from aws_sso_lib.compat import shell_join
 from aws_sso_lib.config import find_instances, SSOInstance
 from botocore.compat import compat_shell_split as shell_split
+from aws_sso_lib.config_file_writer import process_profile_name
 
 AWS_CONFIG_PATH = f'{Path.home()}/.aws/config'
 AWS_CREDENTIAL_PATH = f'{Path.home()}/.aws/credentials'
 AWS_SSO_CACHE_PATH = f'{Path.home()}/.aws/sso/cache'
+AWS_SSO_PROFILE = "aws-sso"
+AWS_SSO_DIR = f".{AWS_SSO_PROFILE}-magic"
+AWS_SSO_CONFIG_PATH = f'{Path.home()}/{AWS_SSO_DIR}/config'
+AWS_SSO_EKS_CONFIG_PATH = f'{Path.home()}/{AWS_SSO_DIR}/eks'
+AWS_SSO_PROFILE_IN_USE = "ProfileInUse"
+AWS_SSO_CONFIG_ALIAS = "AliasAccounts"
+AWS_SSO_EKS_ROLE_NAME_DEFAULT = "replacethis"
 AWS_DEFAULT_REGION = 'us-east-1'
 VERBOSE = True
 
@@ -116,7 +124,7 @@ def get_instance(sso_start_url, sso_region, sso_start_url_vars=None, sso_region_
                 f"No AWS SSO instance matched {specifier.to_str(region=True)} " +
                 f"from {SSOInstance.to_strs(all_instances)}")
         else:
-            raise GetInstanceError("No AWS SSO instance found")
+            raise GetInstanceError("No AWS SSO instance found, run aws-sso-magic configure")
 
     if len(instances) > 1:
         raise GetInstanceError(f"Found {len(instances)} SSO instance, please specify one: {SSOInstance.to_strs(instances)}")
@@ -205,10 +213,29 @@ class Printer:
             first_loop = False
 
 # Check Utils
+
+def _check_flag_combinations(eks, profile_arg):
+    if eks and profile_arg != None:
+        _print_error(f"\nERROR: Not use the flag combination --eks --profile")    
+
+def _check_kubectl():
+    try:
+        kubectl_version = subprocess.run(['kubectl'] + ['version'], capture_output=True).stdout.decode('utf-8')
+        if 'GitVersion:' not in kubectl_version:
+            _print_error('\nkubectl not found. Please install. Exiting.')
+            exit(1)
+        else:
+            print('kubectl found. ')
+    except Exception as e:
+        _print_error(
+            f'\nAn error occured trying to find the kubectl version. Do you have kubectl installed?\n{e}')
+        exit(1)            
+
+
 def _check_aws_v2():
     # validate aws v2
     try:
-        aws_version = subprocess.run(['aws']+ ['--version'], capture_output=True).stdout.decode('utf-8')
+        aws_version = subprocess.run(['aws'] + ['--version'], capture_output=True).stdout.decode('utf-8')
         if 'aws-cli/2' not in aws_version:
             _print_error('\nAWS CLI Version 2 not found. Please install. Exiting.')
             exit(1)
@@ -307,28 +334,42 @@ def get_trim_formatter(account_name_patterns, role_name_patterns, formatter):
 def get_safe_account_name(name):
     return re.sub(r"[\s\[\]]+", "-", name).strip("-")
 
+def _get_profile_name(profile):
+    profile = str(profile).replace('profile ','')
+    return profile 
+
+def _select_profile():
+    config = _read_config(AWS_CONFIG_PATH)
+
+    profiles = []
+    for section in config.sections():
+        x = _get_profile_name(section)
+        profiles.append(x)
+    profiles.sort()
+
+    questions = [
+        inquirer.List(
+            'name',
+            message='Please select an AWS config profile',
+            choices=profiles
+        ),
+    ]
+    answer = inquirer.prompt(questions)
+    return answer['name'] if answer else sys.exit(1)
+
 def get_config_profile_list(configs):
     configure_logging(LOGGER, False)
-    result = PrettyTable()
-    result.field_names = ["Option", "Role"]
-    for c in configs:
-        conf_number = configs.index(c) + 1
-        result.add_row([conf_number, c.profile_name])
-    print (result)
-    try:
-        config_option = int(input('Enter the role option: '))
-        config_option = config_option - 1
-    except ValueError:
-        _print_error('Please enter a number')
-        sys.exit(1)        
-    try:
-        config_profile = configs[config_option].profile_name
-        print(f"\nAWS config profile option selected: {config_profile}")
-    except IndexError as e:
-        _print_error('Option selected not valid')
-        sys.exit(1) 
-    return config_profile
+    answer = _select_profile()
+    return answer
 
+def _get_account_id_profile(path, profile_name):
+    profile_name = f"profile {profile_name}"
+    config = _read_section_configuration(path, profile_name)
+    key_list = list(config.keys())
+    val_list = list(config.values())
+    account_id_position = key_list.index('sso_account_id')
+    account_id = val_list[account_id_position]
+    return account_id
 
 # Credentials Utils
 def _read_config(path):
@@ -358,19 +399,55 @@ class Colour:
     UNDERLINE = '\033[4m'
 
 
-def _set_profile_credentials(profile_name, use_default=False):
+def _set_profile_credentials(profile_name, default_profile):
     profile_opts = _get_aws_profile(profile_name)
     cache_login = _get_sso_cached_login(profile_opts)
     credentials = _get_sso_role_credentials(profile_opts, cache_login)
-
-    if not use_default:
-        _store_aws_credentials(profile_name, profile_opts, credentials)
-    else:
-        _store_aws_credentials('default', profile_opts, credentials)
+    if default_profile == 'default':
+        _store_aws_credentials(default_profile, profile_opts, credentials)
+        _store_aws_credentials(AWS_SSO_PROFILE, profile_opts, credentials)
         _copy_to_default_profile(profile_name)
+        _copy_to_aws_sso_profile(profile_name)
+    else:
+        _store_aws_credentials(profile_name, profile_opts, credentials)
+
+def _get_role_arn(profile_name, role_name):
+    account_id = _get_account_id_profile(AWS_CONFIG_PATH, profile_name)
+    role_arn = f"arn:aws:iam::{account_id}:role/{role_name}"
+    return role_arn
+
+def _create_profilename_child_credentials(parent_profile, profile_name, role_name):
+    profile_name = _get_profile_name(profile_name)
+    role_arn = _get_role_arn(profile_name,role_name)
+    print(f'Adding the profile child [{profile_name}] to the credentials file')
+    config = _read_config(AWS_CREDENTIAL_PATH)
+    if config.has_section(profile_name):
+        config.remove_section(profile_name)
+
+    config.add_section(profile_name)
+    
+    config.set(profile_name , "source_profile", parent_profile)
+    config.set(profile_name , "role_arn", role_arn)
+    _write_config(AWS_CREDENTIAL_PATH, config)
+
+def _copy_to_aws_sso_profile(profile_name):
+    print(f"\nCopying profile [{profile_name}] to [{AWS_SSO_PROFILE}]")
+
+    config = _read_config(AWS_CONFIG_PATH)
+
+    if config.has_section(AWS_SSO_PROFILE):
+        config.remove_section(AWS_SSO_PROFILE)
+
+    config.add_section(AWS_SSO_PROFILE)
+
+    for key, value in config.items(profile_name):
+        config.set(AWS_SSO_PROFILE, key, value)
+
+    _write_config(AWS_CONFIG_PATH, config)
+    print("\nCredentials copied successfully") 
 
 def _copy_to_default_profile(profile_name):
-    print(f'Copying profile [{profile_name}] to [default]')
+    print(f"\nCopying profile [{profile_name}] to [default]")
 
     config = _read_config(AWS_CONFIG_PATH)
 
@@ -467,5 +544,124 @@ def _print_error(message):
     _print_colour(Colour.FAIL, message, always=True)
     sys.exit(1)
 
-def _print_success(message):
-    _print_colour(Colour.OKGREEN, message)    
+def _read_section_configuration(path, section):
+    config = ConfigParser()
+    par    = {}
+    try:
+        config.read(path)
+        par=dict(config.items(section))
+        for p in par:
+            par[p]=par[p].split("#",1)[0].strip()
+        return par        
+    except Exception as e:
+        return par  
+
+def _read_aws_sso_config_file(path, section):
+    config = ConfigParser()
+    par    = {}
+    try:
+        config.read(path)
+        par=dict(config.items(section))
+        for p in par:
+            par[p]=par[p].split("#",1)[0].strip()
+        return par        
+    except Exception as e :
+        return par    
+
+def _set_profile_in_use(profile_name):
+    profile_name = _get_profile_name(profile_name)
+    config = _read_config(AWS_SSO_CONFIG_PATH)
+
+    if config.has_section(AWS_SSO_PROFILE_IN_USE):
+        config.remove_section(AWS_SSO_PROFILE_IN_USE)
+
+    config.add_section(AWS_SSO_PROFILE_IN_USE)
+    
+    config.set(AWS_SSO_PROFILE_IN_USE , "profile", profile_name)
+
+    _write_config(AWS_SSO_CONFIG_PATH, config)
+
+
+def _get_profile_in_use():
+    profile_selected = ""
+    config = _read_aws_sso_config_file(AWS_SSO_CONFIG_PATH, AWS_SSO_PROFILE_IN_USE)
+    res = bool(config)
+    if res:
+        key_list = list(config.keys())
+        val_list = list(config.values())
+        if not 'profile' in key_list :
+            _print_error(f"Please add the [{AWS_SSO_PROFILE_IN_USE}] section with the key profile to the file {AWS_SSO_CONFIG_PATH}")
+        else:
+            position = key_list.index('profile')
+            profile_selected = val_list[position]
+    if profile_selected == "":
+        _print_error(f"\nERROR: Plese run the command aws-sso-magic login and select the profile to use first")
+    return profile_selected
+
+def _role_shortening(profile_name):
+    profile_name = profile_name.replace("administratoraccess", "admin")
+    profile_name = profile_name.replace("readonlyaccess", "readonly")
+    profile_name = profile_name.replace("viewonlyaccess", "viewonly")
+    return profile_name
+
+def _replace_alias(profile_name):
+    partitioned_string = profile_name.partition('-')
+    account_name = partitioned_string[0]
+    config = _read_aws_sso_config_file(AWS_SSO_CONFIG_PATH, AWS_SSO_CONFIG_ALIAS)
+    res = bool(config)
+    if res:
+        key_list = list(config.keys())
+        val_list = list(config.values())
+        if not account_name in key_list :
+            pass
+        else:
+            position = key_list.index(account_name)
+            alias_name = val_list[position]
+            profile_name = profile_name.replace(account_name, alias_name)
+    profile_name = _role_shortening(profile_name)
+    return profile_name
+
+def process_profile_name_formatter(profile_name):
+    profile_name = process_profile_name(profile_name)
+    profile_name = profile_name.replace(".", "-").lower() 
+    profile = _replace_alias (profile_name)
+    return profile
+
+def _create_tool_directory(parent_dir, directory):
+    path = os.path.join(parent_dir, directory)
+    dir_exists = os.path.isdir(path)
+    if not dir_exists:
+        try:
+            os.makedirs(path, exist_ok = True)
+        except OSError as error:
+            _print_error(f"{path} can not be created")
+
+def _create_aws_sso_conf_file(configfile_name):
+    # Check if there is already a configurtion file
+    if not os.path.isfile(configfile_name):
+        # Create the configuration file as it doesn't exist yet
+        cfgfile = open(configfile_name, "w")
+
+        # Add content to the file
+        Config = ConfigParser()
+        Config.write(cfgfile)
+        cfgfile.close()
+        print(f"{configfile_name} file created")
+
+def _create_aws_sso_eks_file(configfile_name):
+    # Check if there is already a configurtion file
+    if not os.path.isfile(configfile_name):
+        # Create the configuration file as it doesn't exist yet
+        proxy_role_name = input('Enter only the name of the proxy role to use by default. Eg: MyAdminRole: ')        
+        if not proxy_role_name:
+            proxy_role_name = AWS_SSO_EKS_ROLE_NAME_DEFAULT
+
+        cfgfile = open(configfile_name, "w")
+
+        # Add content to the file
+        Config = ConfigParser()
+        Config.add_section("default-proxy-role-name")
+        Config.set("default-proxy-role-name", "proxy_role_name", proxy_role_name)
+        Config.write(cfgfile)
+        cfgfile.close()
+        print(f"{configfile_name} file created")
